package com.diagbot.facade;

import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ProductService;
import com.diagbot.entity.ServiceToken;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.ServiceTokenServiceImpl;
import com.diagbot.vo.ServiceTokenVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @Description: 用户端token业务层
 * @author: zhaops
 * @time: 2018/9/18 16:31
 */
@Component
public class ServiceTokenFacade extends ServiceTokenServiceImpl {


    @Autowired
    ProductServiceFacade productServiceFacade;

    /**
     * @Description: 获取资源的url
     * @Author: ztg
     * @Date: 2018/9/18 16:54
     */
    public RespDTO<Boolean> hasPermission(ServiceTokenVo serviceTokenVo) {
        String appkey = serviceTokenVo.getAppkey();
        String secret = serviceTokenVo.getSecret();
        ServiceToken st = this.getServiceToken(appkey, secret);
        if(null == st) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "appkey或secret错误，appkey=【" + appkey + "】," + "secret=【" + secret + "】");
        }
        ProductService ps = productServiceFacade.getById(st.getProductSeviceId());
        if(ps == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "无权限访问!");
        }
        if(!ps.getProductId().equals(serviceTokenVo.getProductId())) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "无权限访问!");
        }
        Date date = new Date();
        if(st.getExpiringDate().getTime() < date.getTime()) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "访问权限已过期!");
        }
        return RespDTO.onSuc(true);
    }
}
